/**
 Demo of how to call the "expat" library on
 an XML file.  I recommend expat 1.0, from
 	http://www.jclark.com/xml/expat.html

This file must be linked with "-lexpat".

Orion Sky Lawlor, olawlor@acm.org, 2006/04/14 (Public Domain)
*/
#include <iostream>
#include <fstream>
#include "expat.h" /* libexpat XML parser */

/** Accumulates data from parsed XML.
  Currently simple, since we just print stuff out,
  but in a real application would be more complex... */
class myStuff {
public:
	/* Number of nested XML tags so far */
	int level;
	/* Put spaces before output stream */
	std::ostream &print(void) {
		for (int i=0;i<level;i++)
			std::cout<<"  ";
		return std::cout;
	}
	myStuff() {level=0;}
};

/** Called on a start tag, like <foo bar="baz"> */
void myStart(myStuff *s,
	const XML_Char *name, /* foo */
	const XML_Char **atts) /* bar = "baz" */
{
	s->print()<<"Starting tag: '"<<name<<"'\n";
	for (int i=0;atts[i];i+=2) {
		s->print()<<"-Attr '"<<
			atts[i]<<"'='"<<atts[i+1]<<"'\n";
	}
	s->level++; /* entering a new tag */
}

/** Called on an end tag, like </foo> */
void myEnd(myStuff *s,
	const XML_Char *name) /* foo */
{
	s->level--; /* leaving a tag */
	s->print()<<"Ending tag: '"<<name<<"'\n";
}

/** Called on some character data */
void myData(myStuff *s,
	const XML_Char *data,int len)
{
	s->print()<<"Character data: '"<<std::string(data,len)<<"'\n";
}

int main() {
	const char *src="example.xml";
	XML_Parser p=XML_ParserCreate(NULL);
	myStuff s;
	XML_SetUserData(p,&s); /* pass s to our routines */
	/* Register our routines to get called as
	  the file is parsed */
	XML_SetElementHandler(p,
		(XML_StartElementHandler)myStart,
		(XML_EndElementHandler)myEnd);
	XML_SetCharacterDataHandler(p,
		(XML_CharacterDataHandler)myData);
	
	/* Parse the file */
	std::ifstream in(src);
	while (in) {
		enum {len=4096};
		char buf[len];
		in.read(buf,len);
		int n=in.gcount(); /* get chars actually read */
		XML_Parse(p,buf,n,0); /* parse n bytes */
	}
	XML_Parse(p,0,0,1); /* end of input */
	return 0;
}
